import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import ij.plugin.*;
import ij.measure.*;

public class Harpoon_Analyzer implements PlugIn {
        static String[] colors = {"Red","Green","Blue","Magenta","Cyan","Yellow","Orange","Black","White"};
        static String color = "Cyan";
        static float tileWidth=16, tileHeight=16, tileWidthOffset=0, tileHeightOffset=0.0001f, threshold=10, border=10;


		
		int xstart, ystart;

        public void run(String arg) {
                if (IJ.versionLessThan("1.43u"))
                        return;
                ImagePlus imp = IJ.getImage();
                boolean notDone = true;
			    //next lines update tileWidth and tileHeight based on imagesize, can use or not
				if ((tileWidth == 16) & (tileHeight == 16)) {
					tileWidth = setWidth(imp, 8);
					tileHeight = setHeight(imp, 8);
				}
                do {
                        if (showDialog())
                        	drawGrid(imp);
                        else {
                        	notDone = false;
							break;
                        }   	
                } while(!gridGood());
        		if(notDone) {
        			if (showDialog2())
        				overthrust(imp);
        		}

        }
                
        void drawGrid(ImagePlus imp) {
                GeneralPath path = new GeneralPath();
                int width = imp.getWidth();
                int height = imp.getHeight();
                float xoff = tileWidthOffset;
                while (true) { // draw vertical lines
                        if (xoff>=width) break;
                        path.moveTo(xoff, 0f);
                        path.lineTo(xoff, height);
                        xoff += tileWidth;
                }
                float yoff = tileHeightOffset;
                while (true) { // draw horizonal lines
                        if (yoff>=height) break;
                        path.moveTo(0f, yoff);
                        path.lineTo(width, yoff);
                        yoff += tileHeight;
                }
                if (path==null)
                        imp.setOverlay(null);
                else
                        imp.setOverlay(path, getColor(), null);
        }

        void overthrust(ImagePlus imp) {
                int width = imp.getWidth();
                int height = imp.getHeight();
                float yoff = tileHeightOffset;
                while (true) { // move down
                        if (yoff>=(height - tileHeight)) break;
                        float xoff = tileWidthOffset;
                        while (true) { // move across
                                if (xoff>=(width - tileWidth)) break;
                                imp.setRoi((int) (xoff + border), (int) (yoff + border), (int) (tileWidth - (2 * border)), (int) (tileHeight - (2 *border)));
                                //in progress
				//threshold = optimizeThreshold(imp);
				IJ.run(imp, "Find Maxima...", "noise=" + threshold + " output=[Point Selection] exclude");
                                IJ.run(imp, "Measure", "");
                                xoff += tileWidth;
                        }
                        yoff += tileHeight;
                }
        }
        
		float optimizeThreshold(ImagePlus imp) {
			//IJ.run(imp, "Find Maxima...", "noise=10 output=Count");
			return 0;
		}
		
        boolean gridGood() {
                GenericDialog gd = new GenericDialog("Ok?");
                gd.showDialog();
                if (gd.wasCanceled()) return false;
                return true;
        }
        
        boolean showDialog() {
                GenericDialog gd = new GenericDialog("Grid");
                gd.addNumericField("Tile Width:", tileWidth, 0);
                gd.addNumericField("Tile Height:", tileHeight, 0);
                gd.addNumericField("X offset:", tileWidthOffset, 0);
                gd.addNumericField("Y offset:", tileHeightOffset, 0);
                gd.addChoice("Color:", colors, color);
                gd.showDialog();
                if (gd.wasCanceled()) return false;
                tileWidth = (float)gd.getNextNumber();
                tileHeight = (float)gd.getNextNumber();
                tileWidthOffset = (float)gd.getNextNumber();
                tileHeightOffset = (float)gd.getNextNumber();
                color = gd.getNextChoice();
                return true;
        }

        boolean showDialog2() {
                GenericDialog gd = new GenericDialog("Overthruster!");
                gd.addNumericField("Threshold:", threshold, 2);
                gd.addNumericField("Border:", border, 2);
                gd.showDialog();
                if (gd.wasCanceled()) return false;
                threshold = (float)gd.getNextNumber();
                border = (float)gd.getNextNumber();
                return true;
        }

        Color getColor() {
                Color c = Color.cyan;
                if (color.equals(colors[0])) c = Color.red;
                else if (color.equals(colors[1])) c = Color.green;
                else if (color.equals(colors[2])) c = Color.blue;
                else if (color.equals(colors[3])) c = Color.magenta;
                else if (color.equals(colors[4])) c = Color.cyan;
                else if (color.equals(colors[5])) c = Color.yellow;
                else if (color.equals(colors[6])) c = Color.orange;
                else if (color.equals(colors[7])) c = Color.black;
                else if (color.equals(colors[8])) c = Color.white;
                return c;
        }
		
		float setHeight(ImagePlus imp, int rows) {
			return imp.getHeight()/rows;
		}
		
		float setWidth(ImagePlus imp, int columns) {
			return imp.getWidth()/columns;
		}
}
